import { Prec } from "@codemirror/state";
import { Awareness, LoroDoc, UndoManager } from "loro-crdt";
import { createCursorLayer, createSelectionLayer, AwarenessPlugin, remoteAwarenessStateField, RemoteAwarenessPlugin, loroCursorTheme, } from "./awareness.js";
import { LoroSyncPluginValue } from "./sync.js";
import { keymap, ViewPlugin } from "@codemirror/view";
import { undoKeyMap, undoManagerStateField, UndoPluginValue } from "./undo.js";
export { undo, redo } from "./undo.js";
export { getTextFromDoc } from "./sync.js";
export const LoroSyncPlugin = (doc) => {
    return ViewPlugin.define((view) => new LoroSyncPluginValue(view, doc));
};
export const LoroAwarenessPlugin = (doc, awareness, user) => {
    return [
        remoteAwarenessStateField,
        createCursorLayer(),
        createSelectionLayer(),
        ViewPlugin.define((view) => new AwarenessPlugin(view, doc, user, awareness)),
        ViewPlugin.define((view) => new RemoteAwarenessPlugin(view, doc, awareness)),
        loroCursorTheme,
    ];
};
export const LoroUndoPlugin = (doc, undoManager) => {
    return [
        undoManagerStateField.init(() => undoManager),
        Prec.high(keymap.of([...undoKeyMap])),
        ViewPlugin.define((view) => new UndoPluginValue(view, doc, undoManager)),
    ];
};
export function LoroExtensions(doc, awareness, undoManager) {
    let extension = [
        ViewPlugin.define((view) => new LoroSyncPluginValue(view, doc))
            .extension,
    ];
    if (undoManager) {
        extension = extension.concat([
            undoManagerStateField.init(() => undoManager),
            Prec.high(keymap.of([...undoKeyMap])),
            ViewPlugin.define((view) => new UndoPluginValue(view, doc, undoManager)).extension,
        ]);
    }
    if (awareness) {
        extension = extension.concat([
            remoteAwarenessStateField,
            createCursorLayer(),
            createSelectionLayer(),
            ViewPlugin.define((view) => new AwarenessPlugin(view, doc, awareness.user, awareness.awareness)),
            ViewPlugin.define((view) => new RemoteAwarenessPlugin(view, doc, awareness.awareness)),
            loroCursorTheme,
        ]);
    }
    return extension;
}
